@@ -218,6 +218,16 @@ class Script(web.Script):
         debug.println(debug.LEVEL_INFO, msg)
         default.Script.onNameChanged(self, event)
 
+    def onSelectedChanged(self, event):
+        """Callback for object:state-changed:selected accessibility events."""
+
+        if super().onSelectedChanged(event):
+            return
+
+        msg = "GECKO: Passing along event to default script"
+        debug.println(debug.LEVEL_INFO, msg)
+        default.Script.onSelectedChanged(self, event)
+
     def onSelectionChanged(self, event):
         """Callback for object:selection-changed accessibility events."""
 
@@ -234,6 +244,12 @@ class Script(web.Script):
         if super().onShowingChanged(event):
             return
 
+        if event.detail1 and self.utilities.isTopLevelChromeAlert(event.source):
+            msg = "GECKO: Event handled: Presenting event source"
+            debug.println(debug.LEVEL_INFO, msg)
+            self.presentObject(event.source)
+            return True
+
         msg = "GECKO: Passing along event to default script"
         debug.println(debug.LEVEL_INFO, msg)
         default.Script.onShowingChanged(self, event)
@@ -268,6 +284,16 @@ class Script(web.Script):
         debug.println(debug.LEVEL_INFO, msg)
         default.Script.onTextSelectionChanged(self, event)
 
+    def onWindowActivated(self, event):
+        """Callback for window:activate accessibility events."""
+
+        if super().onWindowActivated(event):
+            return
+
+        msg = "GECKO: Passing along event to default script"
+        debug.println(debug.LEVEL_INFO, msg)
+        default.Script.onWindowActivated(self, event)
+
     def onWindowDeactivated(self, event):
         """Callback for window:deactivate accessibility events."""
 
