@@ -967,6 +967,8 @@ class Script(default.Script):
             return False
 
         if oldFocus and self.utilities.isZombie(oldFocus):
+            msg = "WEB: Old focus is Zombie: %s. Clearing oldFocus." % oldFocus
+            debug.println(debug.LEVEL_INFO, msg)
             oldFocus = None
 
         caretOffset = 0
@@ -1013,26 +1015,21 @@ class Script(default.Script):
     def onBusyChanged(self, event):
         """Callback for object:state-changed:busy accessibility events."""
 
+        if event.detail1 and self._loadingDocumentContent:
+            msg = "WEB: Ignoring: Already loading document content"
+            debug.println(debug.LEVEL_INFO, msg)
+            return True
+
         if not self.utilities.inDocumentContent(event.source):
             msg = "WEB: Event source is not in document content"
             debug.println(debug.LEVEL_INFO, msg)
             return False
 
-        if not self.utilities.inDocumentContent(orca_state.locusOfFocus):
-            msg = "WEB: Ignoring: Locus of focus is not in document content"
-            debug.println(debug.LEVEL_INFO, msg)
-            return True
-
         if self.utilities.getDocumentForObject(event.source.parent):
             msg = "WEB: Ignoring: Event source is nested document"
             debug.println(debug.LEVEL_INFO, msg)
             return True
 
-        if event.detail1 and self._loadingDocumentContent:
-            msg = "WEB: Ignoring: Already loading document content"
-            debug.println(debug.LEVEL_INFO, msg)
-            return True
-
         self._loadingDocumentContent = event.detail1
 
         obj, offset = self.utilities.getCaretContext()
@@ -1081,7 +1078,7 @@ class Script(default.Script):
             return True
 
         self.updateBraille(obj)
-        if state.contains(pyatspi.STATE_FOCUSABLE):
+        if state.contains(pyatspi.STATE_FOCUSABLE) and not self.utilities.isDocument(obj):
             msg = "WEB: Not doing SayAll due to focusable context obj %s" % obj
             debug.println(debug.LEVEL_INFO, msg)
             speech.speak(self.speechGenerator.generateSpeech(obj))
@@ -1112,12 +1109,6 @@ class Script(default.Script):
             debug.println(debug.LEVEL_INFO, msg)
             return False
 
-        if not self.utilities.inDocumentContent(orca_state.locusOfFocus):
-            msg = "WEB: Event ignored: locusOfFocus (%s) is not in document content" \
-                  % orca_state.locusOfFocus
-            debug.println(debug.LEVEL_INFO, msg)
-            return True
-
         if self._lastCommandWasCaretNav:
             msg = "WEB: Event ignored: Last command was caret nav"
             debug.println(debug.LEVEL_INFO, msg)
@@ -1257,6 +1248,10 @@ class Script(default.Script):
             msg = "WEB: Clearing structural navigation cache for %s" % document
             debug.println(debug.LEVEL_INFO, msg)
             self.structuralNavigation.clearCache(document)
+        else:
+            msg = "WEB: Could not get document for event source"
+            debug.println(debug.LEVEL_INFO, msg)
+            return False
 
         if self.utilities.handleAsLiveRegion(event):
             msg = "WEB: Event to be handled as live region"
@@ -1269,18 +1264,30 @@ class Script(default.Script):
             debug.println(debug.LEVEL_INFO, msg)
             return True
 
+        if self.utilities.isZombie(document):
+            msg = "WEB: Ignoring because %s is zombified." % document
+            debug.println(debug.LEVEL_INFO, msg)
+            return True
+
+        try:
+            docIsBusy = document.getState().contains(pyatspi.STATE_BUSY)
+        except:
+            docIsBusy = False
+            msg = "WEB: Exception getting state of %s" % document
+            debug.println(debug.LEVEL_INFO, msg)
+        if docIsBusy:
+            msg = "WEB: Ignoring because %s is busy." % document
+            debug.println(debug.LEVEL_INFO, msg)
+            return True
+
         if not event.any_data or self.utilities.isZombie(event.any_data):
             msg = "WEB: Ignoring because any data is null or zombified."
             debug.println(debug.LEVEL_INFO, msg)
             return True
 
-        if not self.utilities.inDocumentContent(event.source):
-            msg = "WEB: Event source is not in document content"
-            debug.println(debug.LEVEL_INFO, msg)
-            return False
-
         obj, offset = self.utilities.getCaretContext()
-        if obj and self.utilities.isZombie(obj):
+        contextDocument = self.utilities.getDocumentForObject(obj)
+        if obj and document == contextDocument and self.utilities.isZombie(obj):
             replicant = self.utilities.findReplicant(event.source, obj)
             if replicant:
                 # Refrain from actually touching the replicant by grabbing
@@ -1426,9 +1433,34 @@ class Script(default.Script):
 
         return True
 
+    def onSelectedChanged(self, event):
+        """Callback for object:state-changed:selected accessibility events."""
+
+        if self.utilities.eventIsChromeAutocompleteNoise(event):
+            msg = "WEB: Ignoring event believed to be chrome autocomplete noise"
+            debug.println(debug.LEVEL_INFO, msg)
+            return True
+
+        if not self.utilities.inDocumentContent(event.source):
+            msg = "WEB: Event source is not in document content"
+            debug.println(debug.LEVEL_INFO, msg)
+            return False
+
+        if orca_state.locusOfFocus != event.source:
+            msg = "WEB: Ignoring because event source is not locusOfFocus"
+            debug.println(debug.LEVEL_INFO, msg)
+            return True
+
+        return False
+
     def onSelectionChanged(self, event):
         """Callback for object:selection-changed accessibility events."""
 
+        if self.utilities.eventIsChromeAutocompleteNoise(event):
+            msg = "WEB: Ignoring event believed to be chrome autocomplete noise"
+            debug.println(debug.LEVEL_INFO, msg)
+            return True
+
         if not self.utilities.inDocumentContent(event.source):
             msg = "WEB: Event source is not in document content"
             debug.println(debug.LEVEL_INFO, msg)
@@ -1462,6 +1494,11 @@ class Script(default.Script):
     def onTextDeleted(self, event):
         """Callback for object:text-changed:delete accessibility events."""
 
+        if self.utilities.isZombie(event.source):
+            msg = "WEB: Event source is Zombie"
+            debug.println(debug.LEVEL_INFO, msg)
+            return True
+
         if self.utilities.eventIsStatusBarNoise(event):
             msg = "WEB: Ignoring event believed to be status bar noise"
             debug.println(debug.LEVEL_INFO, msg)
@@ -1508,6 +1545,11 @@ class Script(default.Script):
     def onTextInserted(self, event):
         """Callback for object:text-changed:insert accessibility events."""
 
+        if self.utilities.isZombie(event.source):
+            msg = "WEB: Event source is Zombie"
+            debug.println(debug.LEVEL_INFO, msg)
+            return True
+
         if self.utilities.eventIsStatusBarNoise(event):
             msg = "WEB: Ignoring event believed to be status bar noise"
             debug.println(debug.LEVEL_INFO, msg)
@@ -1616,6 +1658,11 @@ class Script(default.Script):
 
         return False
 
+    def onWindowActivated(self, event):
+        """Callback for window:activate accessibility events."""
+
+        return False
+
     def onWindowDeactivated(self, event):
         """Callback for window:deactivate accessibility events."""
 
