@@ -56,6 +56,7 @@ class Utilities(script_utilities.Utilities):
         self._isOffScreenLabel = {}
         self._hasNoSize = {}
         self._hasLongDesc = {}
+        self._hasUselessCanvasDescendant = {}
         self._isClickableElement = {}
         self._isAnchor = {}
         self._isLandmark = {}
@@ -64,6 +65,7 @@ class Utilities(script_utilities.Utilities):
         self._isNonNavigablePopup = {}
         self._isNonEntryTextWidget = {}
         self._isUselessImage = {}
+        self._isParentOfNullChild = {}
         self._inferredLabels = {}
         self._roleDescription = {}
         self._text = {}
@@ -95,6 +97,7 @@ class Utilities(script_utilities.Utilities):
         self._isOffScreenLabel = {}
         self._hasNoSize = {}
         self._hasLongDesc = {}
+        self._hasUselessCanvasDescendant = {}
         self._isClickableElement = {}
         self._isAnchor = {}
         self._isLandmark = {}
@@ -103,6 +106,7 @@ class Utilities(script_utilities.Utilities):
         self._isNonNavigablePopup = {}
         self._isNonEntryTextWidget = {}
         self._isUselessImage = {}
+        self._isParentOfNullChild = {}
         self._inferredLabels = {}
         self._roleDescription = {}
         self._tag = {}
@@ -118,10 +122,25 @@ class Utilities(script_utilities.Utilities):
         self._currentAttrs = {}
         self._text = {}
 
+    def isDocument(self, obj):
+        roles = [pyatspi.ROLE_DOCUMENT_FRAME, pyatspi.ROLE_DOCUMENT_WEB, pyatspi.ROLE_EMBEDDED]
+
+        try:
+            rv = obj.getRole() in roles
+        except:
+            msg = "WEB: Exception getting role for %s" % obj
+            debug.println(debug.LEVEL_INFO, msg)
+            rv = False
+
+        return rv
+
     def inDocumentContent(self, obj=None):
         if not obj:
             obj = orca_state.locusOfFocus
 
+        if self.isDocument(obj):
+            return True
+
         rv = self._inDocumentContent.get(hash(obj))
         if rv is not None:
             return rv
@@ -135,20 +154,15 @@ class Utilities(script_utilities.Utilities):
         if not obj:
             return None
 
-        roles = [pyatspi.ROLE_DOCUMENT_FRAME, pyatspi.ROLE_DOCUMENT_WEB, pyatspi.ROLE_EMBEDDED]
-
-        def isDocument(x):
-            try:
-                return x and x.getRole() in roles
-            except:
-                msg = "WEB: Exception getting role for %s" % x
-                debug.println(debug.LEVEL_INFO, msg)
-                return False
-
-        if isDocument(obj):
+        if self.isDocument(obj):
+            msg = "WEB: %s is document" % obj
+            debug.println(debug.LEVEL_INFO, msg)
             return obj
 
-        return pyatspi.findAncestor(obj, isDocument)
+        document = pyatspi.findAncestor(obj, self.isDocument)
+        msg = "WEB: Document for %s is %s" % (obj, document)
+        debug.println(debug.LEVEL_INFO, msg)
+        return document
 
     def _getDocumentsEmbeddedBy(self, frame):
         isEmbeds = lambda r: r.getRelationType() == pyatspi.RELATION_EMBEDS
@@ -616,6 +630,9 @@ class Utilities(script_utilities.Utilities):
         return rv
 
     def queryNonEmptyText(self, obj, excludeNonEntryTextWidgets=True):
+        if not obj:
+            return None
+
         if hash(obj) in self._text:
             return self._text.get(hash(obj))
 
@@ -631,12 +648,15 @@ class Utilities(script_utilities.Utilities):
         if not self.isLiveRegion(obj):
             doNotQuery = [pyatspi.ROLE_TABLE_ROW,
                           pyatspi.ROLE_TOOL_BAR]
-            if rv and obj.getRole() in doNotQuery:
+            role = obj.getRole()
+            if rv and role in doNotQuery:
                 rv = None
             if rv and excludeNonEntryTextWidgets and self.isNonEntryTextWidget(obj):
                 rv = None
             if rv and (self.isHidden(obj) or self.isOffScreenLabel(obj)):
                 rv = None
+            if rv and role == pyatspi.ROLE_LINK and self.hasUselessCanvasDescendant(obj):
+                rv = None
 
         self._text[hash(obj)] = rv
         return rv
@@ -822,11 +842,13 @@ class Utilities(script_utilities.Utilities):
                 math = self.getMathAncestor(obj)
             return [[math, 0, 1, '']]
 
-        if obj.getRole() == pyatspi.ROLE_INTERNAL_FRAME and obj.childCount == 1:
+        role = obj.getRole()
+        if role == pyatspi.ROLE_INTERNAL_FRAME and obj.childCount == 1:
             return self._getContentsForObj(obj[0], 0, boundary)
 
         string, start, end = self._getTextAtOffset(obj, offset, boundary)
-        if not string:
+        # Check for ROLE_SECTION due to https://bugzilla.mozilla.org/show_bug.cgi?id=1210630
+        if not string or (self.isLandmark(obj) and role != pyatspi.ROLE_SECTION):
             return [[obj, start, end, string]]
 
         stringOffset = offset - start
@@ -1165,7 +1187,12 @@ class Utilities(script_utilities.Utilities):
         debug.println(debug.LEVEL_INFO, msg)
 
         if obj and self.isZombie(obj):
-            msg = "WEB: Current context obj %s is zombie" % obj
+            msg = "WEB: Current context obj %s is zombie. Clearing cache." % obj
+            debug.println(debug.LEVEL_INFO, msg)
+            self.clearCachedObjects()
+
+            obj, offset = self.getCaretContext()
+            msg = "WEB: Now Current context is: %s, %i" % (obj, offset)
             debug.println(debug.LEVEL_INFO, msg)
 
         line = self.getLineContentsAtOffset(obj, offset, layoutMode, useCache)
@@ -1205,7 +1232,12 @@ class Utilities(script_utilities.Utilities):
         debug.println(debug.LEVEL_INFO, msg)
 
         if obj and self.isZombie(obj):
-            msg = "WEB: Current context obj %s is zombie" % obj
+            msg = "WEB: Current context obj %s is zombie. Clearing cache." % obj
+            debug.println(debug.LEVEL_INFO, msg)
+            self.clearCachedObjects()
+
+            obj, offset = self.getCaretContext()
+            msg = "WEB: Now Current context is: %s, %i" % (obj, offset)
             debug.println(debug.LEVEL_INFO, msg)
 
         line = self.getLineContentsAtOffset(obj, offset, layoutMode, useCache)
@@ -1668,7 +1700,7 @@ class Utilities(script_utilities.Utilities):
 
             if (self.isTextBlockElement(obj) and not string.strip()) \
                or self.isAnchor(obj) \
-               or self.hasNoSize(obj) \
+               or (self.hasNoSize(obj) and not string.strip()) \
                or self.isOffScreenLabel(obj) \
                or self.isUselessImage(obj) \
                or self.isLabellingContents(x, contents):
@@ -1681,6 +1713,9 @@ class Utilities(script_utilities.Utilities):
 
             return True
 
+        if len(contents) == 1:
+            return contents
+
         return list(filter(_include, contents))
 
     def needsSeparator(self, lastChar, nextChar):
@@ -1869,6 +1904,21 @@ class Utilities(script_utilities.Utilities):
         self._isAnchor[hash(obj)] = rv
         return rv
 
+    def isChromeAlert(self, obj):
+        if not (obj and obj.getRole() == pyatspi.ROLE_ALERT):
+            return False
+
+        if self.inDocumentContent(obj):
+            return False
+
+        return True
+
+    def isTopLevelChromeAlert(self, obj):
+        if not self.isChromeAlert(obj):
+            return False
+
+        return obj.parent.getRole() == pyatspi.ROLE_FRAME
+
     def isClickableElement(self, obj):
         if not (obj and self.inDocumentContent(obj)):
             return False
@@ -1970,6 +2020,27 @@ class Utilities(script_utilities.Utilities):
         self._isNonNavigablePopup[hash(obj)] = rv
         return rv
 
+    def hasUselessCanvasDescendant(self, obj):
+        if not (obj and self.inDocumentContent(obj)):
+            return False
+
+        rv = self._hasUselessCanvasDescendant.get(hash(obj))
+        if rv is not None:
+            return rv
+
+        isCanvas = lambda x: x and x.getRole() == pyatspi.ROLE_CANVAS
+        try:
+            canvases = pyatspi.findAllDescendants(obj, isCanvas)
+        except:
+            msg = "WEB: Exception getting descendant canvases of %s" % obj
+            debug.println(debug.LEVEL_INFO, msg)
+            rv = False
+        else:
+            rv = len(list(filter(self.isUselessImage, canvases))) > 0
+
+        self._hasUselessCanvasDescendant[hash(obj)] = rv
+        return rv
+
     def isUselessImage(self, obj):
         if not (obj and self.inDocumentContent(obj)):
             return False
@@ -1979,7 +2050,7 @@ class Utilities(script_utilities.Utilities):
             return rv
 
         rv = True
-        if obj.getRole() != pyatspi.ROLE_IMAGE:
+        if obj.getRole() not in [pyatspi.ROLE_IMAGE, pyatspi.ROLE_CANVAS]:
             rv = False
         if rv and (obj.name or obj.description or obj.childCount):
             rv = False
@@ -1997,10 +2068,35 @@ class Utilities(script_utilities.Utilities):
                 width, height = image.getImageSize()
                 if width > 25 and height > 25:
                     rv = False
+        if rv and 'Text' in pyatspi.listInterfaces(obj):
+            rv = self.queryNonEmptyText(obj) is None
 
         self._isUselessImage[hash(obj)] = rv
         return rv
 
+    def isParentOfNullChild(self, obj):
+        if not (obj and self.inDocumentContent(obj)):
+            return False
+
+        rv = self._isParentOfNullChild.get(hash(obj))
+        if rv is not None:
+            return rv
+
+        rv = False
+        try:
+            childCount = obj.childCount
+        except:
+            msg = "WEB: Exception getting childCount for %s" % obj
+            debug.println(debug.LEVEL_INFO, msg)
+            childCount = 0
+        if childCount and obj[0] is None:
+            msg = "ERROR: %s reports %i children, but obj[0] is None" % (obj, childCount)
+            debug.println(debug.LEVEL_INFO, msg)
+            rv = True
+
+        self._isParentOfNullChild[hash(obj)] = rv
+        return rv
+
     def hasLongDesc(self, obj):
         if not (obj and self.inDocumentContent(obj)):
             return False
@@ -2111,7 +2207,13 @@ class Utilities(script_utilities.Utilities):
 
         eType = event.type
         if eType.startswith("object:text-") or eType.endswith("accessible-name"):
-            return event.source.getRole() == pyatspi.ROLE_STATUS_BAR
+            try:
+                role = event.source.getRole()
+            except:
+                msg = "WEB: Exception getting role for %s" % event.source
+                debug.println(debug.LEVEL_INFO, msg)
+            else:
+                return role == pyatspi.ROLE_STATUS_BAR
 
         return False
 
@@ -2136,6 +2238,38 @@ class Utilities(script_utilities.Utilities):
 
         return False
 
+    def eventIsChromeAutocompleteNoise(self, event):
+        if self.inDocumentContent(event.source):
+            return False
+
+        selection = ["object:selection-changed", "object:state-changed:selected"]
+        if not event.type in selection:
+            return False
+
+        try:
+            focusRole = orca_state.locusOfFocus.getRole()
+            focusState = orca_state.locusOfFocus.getState()
+        except:
+            msg = "WEB: Exception getting role and state for %s" % orca_state.locusOfFocus
+            debug.println(debug.LEVEL_INFO, msg)
+            return False
+
+        try:
+            role = event.source.getRole()
+        except:
+            msg = "WEB: Exception getting role for %s" % event.source
+            debug.println(debug.LEVEL_INFO, msg)
+            return False
+
+        if role in [pyatspi.ROLE_MENU, pyatspi.ROLE_MENU_ITEM] \
+           and focusRole == pyatspi.ROLE_ENTRY \
+           and focusState.contains(pyatspi.STATE_FOCUSED):
+            lastKey, mods = self.lastKeyAndModifiers()
+            if lastKey not in ["Down", "Up"]:
+                return True
+
+        return False
+
     def textEventIsDueToInsertion(self, event):
         if not event.type.startswith("object:text-"):
             return False
@@ -2248,9 +2382,14 @@ class Utilities(script_utilities.Utilities):
         try:
             extents = obj.queryComponent().getExtents(0)
         except:
+            msg = "WEB: Exception getting extents for %s" % obj
+            debug.println(debug.LEVEL_INFO, msg)
             rv = True
         else:
             rv = not (extents.width and extents.height)
+            if rv:
+                msg = "WEB: %s has no size %s" % (obj, extents)
+                debug.println(debug.LEVEL_INFO, msg)
 
         self._hasNoSize[hash(obj)] = rv
         return rv
@@ -2265,12 +2404,16 @@ class Utilities(script_utilities.Utilities):
             msg = "WEB: Exception getting childCount for %s" % obj
             debug.println(debug.LEVEL_INFO, msg)
             return True
-        if not childCount:
+        if not childCount or self.isParentOfNullChild(obj):
             return True
 
         if self.isHidden(obj) or self.isOffScreenLabel(obj):
             return True
 
+        role = obj.getRole()
+        if role == pyatspi.ROLE_LINK and self.hasUselessCanvasDescendant(obj):
+            return True
+
         if self.isTextBlockElement(obj):
             return False
 
@@ -2285,7 +2428,7 @@ class Utilities(script_utilities.Utilities):
                         pyatspi.ROLE_TOOL_TIP,
                         pyatspi.ROLE_TREE,
                         pyatspi.ROLE_TREE_TABLE]
-        return obj.getRole() in doNotDescend
+        return role in doNotDescend
 
     def _searchForCaretContext(self, obj):
         contextObj, contextOffset = None, -1
@@ -2422,7 +2565,7 @@ class Utilities(script_utilities.Utilities):
             elif not self.doNotDescendForCaret(obj) and obj.childCount:
                 return self.findNextCaretInOrder(obj[0], -1)
             elif offset < 0 and not self.isTextBlockElement(obj) and not self.hasNoSize(obj) \
-                 and not self.isUselessImage(obj):
+                 and not self.isUselessImage(obj) and not self.isParentOfNullChild(obj):
                 return obj, 0
 
         # If we're here, start looking up the the tree, up to the document.
@@ -2457,7 +2600,7 @@ class Utilities(script_utilities.Utilities):
                 msg = "WEB: Exception getting childCount for %s" % parent
                 debug.println(debug.LEVEL_INFO, msg)
             else:
-                if 0 <= index < parentChildCount:
+                if 0 < index < parentChildCount:
                     return self.findNextCaretInOrder(parent[index], -1)
             obj = parent
 
@@ -2486,7 +2629,7 @@ class Utilities(script_utilities.Utilities):
             elif not self.doNotDescendForCaret(obj) and obj.childCount:
                 return self.findPreviousCaretInOrder(obj[obj.childCount - 1], -1)
             elif offset < 0 and not self.isTextBlockElement(obj) and not self.hasNoSize(obj) \
-                 and not self.isUselessImage(obj):
+                 and not self.isUselessImage(obj) and not self.isParentOfNullChild(obj):
                 return obj, 0
 
         # If we're here, start looking up the the tree, up to the document.
