@@ -65,6 +65,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                                pyatspi.ROLE_MENU_BAR,
                                pyatspi.ROLE_TOOL_BAR]
         args['skipRoles'] = [pyatspi.ROLE_PARAGRAPH,
+                             pyatspi.ROLE_LABEL,
                              pyatspi.ROLE_LIST_ITEM,
                              pyatspi.ROLE_TEXT]
 
@@ -141,6 +142,11 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if self._script.utilities.isTextBlockElement(obj):
             return []
 
+        if self._script.utilities.inDocumentContent(obj) and obj.name:
+            result = [obj.name]
+            result.extend(self.voice(speech_generator.DEFAULT))
+            return result
+
         return super()._generateLabelOrName(obj, **args)
 
     def _generateLabel(self, obj, **args):
@@ -270,6 +276,9 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                 result.append(messages.IMAGE_MAP_LINK)
                 result.extend(acss)
             else:
+                if self._script.utilities.hasUselessCanvasDescendant(obj):
+                    result.append(self.getLocalizedRoleName(obj, pyatspi.ROLE_IMAGE))
+                    result.extend(acss)
                 result.append(self.getLocalizedRoleName(obj, role))
                 result.extend(acss)
 
